/* 
   Find all anagram groups in a dictionary, 
   and print them to standard output stream

D.R. Musser, G.J. Derge, and Atul Saini, STL Tutorial and Reference Guide, 
Second Edition: C++ Programming with the Standard Template Library
(Boston, MA: Addison-Wesley, 2001)
*/

#include "stdafx.h"
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <iterator>
using namespace std; 
#include "ps.h" // Definitions of PS, firstLess, firstEqual

int main()
{
  cout << "Anagram group finding program:\n"
       << "finds all anagram groups in a dictionary.\n\n" 
       << flush;

  cout << "First, enter the name of the file containing\n"
       << "the dictionary: " << flush;

  string dictionary_name; 
  cin >> dictionary_name;	// diction.txt

  ifstream ifs(dictionary_name.c_str());
  if (!ifs.is_open()) {
    cout << "Eh? Could not open file named "
         << dictionary_name << endl;
    exit(1);
  }

  cout << "\nReading the dictionary ..." << flush;
  typedef istream_iterator<string> string_input;
   
  vector<PS> word_pairs;
  
  copy(string_input(ifs), string_input(), 
       back_inserter(word_pairs));
  
  cout << "\nSearching " << word_pairs.size() 
    << " words for anagram groups..." << flush;

  sort(word_pairs.begin(), word_pairs.end(), firstLess);
  
  vector<PS>::const_iterator j = word_pairs.begin(),
                             finis = word_pairs.end(), k;
  
  cout << "\n\nThe anagram groups are:" << endl;
  
  bool again = true;
  while (again) 
  {
    j = adjacent_find(j, finis, firstEqual);
		// Searches for two adjacent elements that are either equal or satisfy a specified condition
  
	if (j == finis) 
		again = false;
	else
	{
		k = find_if(j + 1, finis, not1(bind1st(firstEqual, *j)));
			// not1 - Returns the complement of a unary predicate
			// bind1st - A helper template function that creates an adaptor
			//           to convert a binary function object into a unary function object 
			//           by binding the first argument of the binary function to a specified value
		cout << "  ";
		copy(j, k, ostream_iterator<string>(cout, " "));
		cout << endl;
		j = k;
	}
  }
  return 0;     
}
